@@ -18,6 +18,7 @@ __all__ = [
     "matrix_rank",
     "slogdet",
     "trace",
+    "diagonal",
     "solve",
     "inv",
 ]
@@ -462,13 +463,13 @@ def qr(da, dims=None, *, mode="reduced", out_append="2", **kwargs):  # pragma: n
         dims = _attempt_default_dims("qr", da.dims)
     m_dim, n_dim = dims
     m, n = len(da[m_dim]), len(da[n_dim])
-    k = min(m, n)
+    k, k_dim = (m, m_dim) if n >= m else (n, n_dim)
     mode = mode.lower()
     if mode == "reduced":
-        if m == k:
-            out_dims = [[m_dim, m_dim + out_append], [m_dim, n_dim]]
-        else:
-            out_dims = [[m_dim, n_dim], [n_dim, n_dim + out_append]]
+        out_dims = [
+            [m_dim, k_dim + (out_append if k_dim == m_dim else "")],
+            [k_dim, n_dim + (out_append if k_dim == n_dim else "")],
+        ]
     elif mode == "complete":
         out_dims = [[m_dim, m_dim + out_append], [m_dim, n_dim]]
     elif mode == "r":
@@ -499,8 +500,7 @@ def svd(
         dims = _attempt_default_dims("svd", da.dims)
     m_dim, n_dim = dims
     m, n = len(da[m_dim]), len(da[n_dim])
-    k, k_dim = (m, m_dim) if m >= n else (n, n_dim)
-    k_dim = m_dim if k == m else n_dim
+    k, k_dim = (m, m_dim) if m <= n else (n, n_dim)
     s_dims = [k_dim]
     if full_matrices:
         u_dims = [m_dim, m_dim + out_append]
@@ -662,6 +662,25 @@ def trace(da, dims=None, *, offset=0, dtype=None, out=None, **kwargs):
     return xr.apply_ufunc(np.trace, da, input_core_dims=[dims], kwargs=trace_kwargs, **kwargs)
 
 
+def diagonal(da, dims=None, *, offset=0, **kwargs):
+    """Wrap :func:`numpy.diagonal`.
+
+    Usage examples of all arguments is available at the :ref:`linalg_tutorial` page.
+    """
+    if dims is None:
+        dims = _attempt_default_dims("diagonal", da.dims)
+    diagonal_kwargs = dict(offset=offset, axis1=-2, axis2=-1)
+    out_dims = [dims[0] if offset == 0 else "diag_id"]
+    return xr.apply_ufunc(
+        np.diagonal,
+        da,
+        input_core_dims=[dims],
+        output_core_dims=[out_dims],
+        kwargs=diagonal_kwargs,
+        **kwargs,
+    )
+
+
 def solve(da, db, dims=None, **kwargs):
     """Wrap :func:`numpy.linalg.solve`.
 
