@@ -6,8 +6,9 @@ from collections.abc import Sequence
 import numpy as np
 import xarray as xr
 from numpy.linalg import LinAlgError
-from scipy import stats
+from scipy import special, stats
 
+from . import _remove_indexes_to_reduce
 from .linalg import cholesky, eigh
 
 __all__ = [
@@ -91,7 +92,15 @@ def _asdataarray(x_or_q, dim_name):
     """
     if isinstance(x_or_q, xr.DataArray):
         return x_or_q
-    return xr.DataArray(np.asarray(x_or_q), dims=[dim_name], coords={dim_name: np.asarray(x_or_q)})
+    x_or_q_ary = np.asarray(x_or_q)
+    if x_or_q_ary.ndim == 0:
+        return xr.DataArray(x_or_q_ary)
+    if x_or_q_ary.ndim == 1:
+        return xr.DataArray(x_or_q_ary, dims=[dim_name], coords={dim_name: np.asarray(x_or_q)})
+    raise ValueError(
+        "To evaluate distribution methods on data with >=2 dims,"
+        " the input needs to be a xarray.DataArray"
+    )
 
 
 def _wrap_method(method):
@@ -401,7 +410,7 @@ def _apply_nonreduce_func(func, da, dims, kwargs, func_kwargs=None):
         dims = get_default_dims(da.dims)
     if not isinstance(dims, str):
         aux_dim = f"__aux_dim__:{','.join(dims)}"
-        da = da.stack({aux_dim: dims})
+        da = _remove_indexes_to_reduce(da, dims).stack({aux_dim: dims})
         core_dims = [aux_dim]
         unstack = True
     else:
@@ -415,7 +424,7 @@ def _apply_nonreduce_func(func, da, dims, kwargs, func_kwargs=None):
         **kwargs,
     )
     if unstack:
-        return out_da.unstack(aux_dim)
+        return _remove_indexes_to_reduce(out_da.unstack(aux_dim), dims).reindex_like(da)
     return out_da
 
 
@@ -429,7 +438,7 @@ def _apply_reduce_func(func, da, dims, kwargs, func_kwargs=None):
         dims = get_default_dims(da.dims)
     if not isinstance(dims, str):
         aux_dim = f"__aux_dim__:{','.join(dims)}"
-        da = da.stack({aux_dim: dims})
+        da = _remove_indexes_to_reduce(da, dims).stack({aux_dim: dims}, create_index=False)
         core_dims = [aux_dim]
     else:
         core_dims = [dims]
@@ -533,6 +542,54 @@ def skew(da, dims=None, *, bias=True, nan_policy=None, **kwargs):
     return _apply_reduce_func(stats.skew, da, dims, kwargs, skew_kwargs)
 
 
+def logsumexp(da, dims=None, *, b=True, return_sign=False, **kwargs):
+    """Wrap and extend :func:`scipy.special.logsumexp`.
+
+    Usage examples available at :ref:`stats_tutorial`
+    """
+    if dims is None:
+        dims = get_default_dims(da.dims)
+    if not isinstance(dims, str):
+        core_dims = dims
+        axis = tuple(-1 - i for i in reversed(range(len(dims))))
+    else:
+        core_dims = [dims]
+        axis = -1
+
+    if return_sign:
+        out_dims = [[], []]
+    else:
+        out_dims = [[]]
+
+    b_dims = []
+    b_dims_to_keep = []
+    if isinstance(b, xr.DataArray):
+        b_dims_to_keep = [d for d in b.dims if d in da.dims and d not in core_dims]
+        b_dims = [d for d in b.dims if d in core_dims]
+        out_dims[0].extend(b_dims_to_keep)
+
+    try:
+        return xr.apply_ufunc(
+            lambda a, b, **kwargs: special.logsumexp(a, b=b, **kwargs),
+            da,
+            b,
+            input_core_dims=[b_dims_to_keep + core_dims, b_dims_to_keep + b_dims],
+            output_core_dims=out_dims,
+            kwargs=dict(return_sign=return_sign, axis=axis),
+            **kwargs,
+        )
+    except ValueError:
+        out_dims[0] = []
+        return xr.apply_ufunc(
+            lambda a, b, **kwargs: special.logsumexp(a, b=b, **kwargs),
+            *xr.broadcast(da, b),
+            input_core_dims=[core_dims, core_dims],
+            output_core_dims=out_dims,
+            kwargs=dict(return_sign=return_sign, axis=axis),
+            **kwargs,
+        )
+
+
 def median_abs_deviation(da, dims=None, *, center=None, scale=1, nan_policy=None, **kwargs):
     """Wrap and extend :func:`scipy.stats.median_abs_deviation`.
 
@@ -576,7 +633,7 @@ def median_abs_deviation(da, dims=None, *, center=None, scale=1, nan_policy=None
         dims = get_default_dims(da.dims)
     if not isinstance(dims, str):
         aux_dim = f"__aux_dim__:{','.join(dims)}"
-        da = da.stack({aux_dim: dims})
+        da = _remove_indexes_to_reduce(da, dims).stack({aux_dim: dims})
         core_dims = [aux_dim]
     else:
         core_dims = [dims]
